@@ -1,11 +1,7 @@
 #' @name Deriv
 #' @title Symbollic differentiation of an expression or function
-#' @aliases Deriv drule
+#' @aliases Deriv drule qlist
 #' @concept symbollic differentiation
-# \usage{
-# Deriv(f, x=if (is.function(f)) names(formals(f)) else all.vars(if (is.character(f)) parse(text=f) else f), env=if (is.function(f)) environment(f) else parent.frame(), use.D=FALSE, cache.exp=TRUE)
-# }
-#' 
 #' 
 #' @param f An expression or function to be differentiated.
 #'  f can be \itemize{
@@ -31,6 +27,8 @@
 #'  final expression must be optimized with cached subexpressions.
 #'  If enabled, repeated calculations are made only once and their
 #'  results stored in cache variables which are then reused.
+#' @param ... (in \code{qlist()}) is a suite of named unevaluated expressions.
+#'  It is used to add derivative rules to \code{drule} environment.
 #' 
 #' @return \itemize{
 #'  \item a function if \code{f} is a function
@@ -73,7 +71,9 @@
 #'  }
 #'  \item It's easy to add custom entries to the derivatives table, e.g.
 #'   
-#'   \code{drule[["cos"]] <- list(-._d1*sin(._1))}
+#'   \code{drule[["cos"]] <- qlist(x=-sin(x))}
+#'   
+#'   The chain rule will be automatically applied if needed.
 #'  \item The output is an executable function, which makes it suitable
 #'      for use in optimization problems.
 #'  \item Compound functions (i.e. piece-wise functions based on if-else operator) can
@@ -85,16 +85,17 @@
 #' As their names indicate, they contain tables of derivative and
 #' simplification rules.
 #' To see the list of defined rules do \code{ls(drule)}.
-#' To add your own derivative rule for a function called say \code{sinpi(x)} calculating sin(pi*x), do \code{drule[["sinpi"]] <- list(quote(pi*._d1*cospi(._1)))}.
-#' Here, "._1" stands for the "first arguments", "._d1" for the first derivative of the first arguments. For a function that might have more than one argument,
-#' e.g. \code{log(x, base=exp(1))}, the drule entry must be a list with one rule
-#' per number of possible arguments. See \code{drule$log} for an example to follow.
-#' After adding \code{sinpi} you can differentiate expressions like \code{Deriv(~ sinpi(x^2), "x")}
+#' To add your own derivative rule for a function called say \code{sinpi(x)} calculating sin(pi*x), do \code{drule[["sinpi"]] <- qlist(x=pi*cospi(x))}.
+#' Here, "x" stands for the first and unique argument in \code{sinpi()} definition. For a function that might have more than one argument,
+#' e.g. \code{log(x, base=exp(1))}, the drule entry must be a list with a named rule
+#' per argument. See \code{drule$log} for an example to follow.
+#' After adding \code{sinpi} you can differentiate expressions like \code{Deriv(~ sinpi(x^2), "x")}. The chain rule will automatically apply.
 #' 
 #' NB. In \code{abs()} and \code{sign()} function, singularity treatment at point 0 is left to user's care.
 #' 
 #' NB2. In Bessel functions, derivatives are calculated only by the first argument,
 #'      not by the \code{nu} argument which is supposed to be constant.
+#' @author Andrew Clausen (original version) and Serguei Sokol (maintainer)
 #' @examples
 #'
 #' \dontrun{f <- function(x) x^2}
@@ -115,7 +116,7 @@
 #' \dontrun{Deriv(~ f(x, y^2), "y")}
 #' # -(2 * (y * sin(x) * sin(y^2)))
 #' 
-#' \dontrun{Deriv(quote(f(x, y^2)), c("x", "y"))}
+#' \dontrun{Deriv(quote(f(x, y^2)), c("x", "y"), cache.exp=FALSE)}
 #' # c(x = cos(x) * cos(y^2), y = -(2 * (y * sin(x) * sin(y^2))))
 #' 
 #' \dontrun{Deriv(expression(sin(x^2) * y), "x")}
@@ -147,8 +148,15 @@
 #' #   -(0.5 * (.d_x * exp(-(0.5 * d))))
 #' #}
 #' 
+#' # Custom derivation rule
+#' \dontrun{
+#'   myfun <- function(x, y=TRUE) NULL # do something usefull
+#'   dmyfun <- function(x, y=TRUE) NULL # myfun derivative by x.
+#'   drule[["myfun"]] <- qlist(x=dmyfun(x, y), y=NULL) # y is just a logical
+#'   Deriv(myfun(z^2, FALSE), "z")
+#'   # 2 * (z * dmyfun(z^2, FALSE))
+#' }
 
-# This wrapper of Deriv_ returns an appropriate expression (wrapped up "properly") depending on the type of argument to differentiate
 Deriv <- function(f, x=if (is.function(f)) names(formals(f)) else all.vars(if (is.character(f)) parse(text=f) else f), env=if (is.function(f)) environment(f) else parent.frame(), use.D=FALSE, cache.exp=TRUE) {
 	tf <- try(f, silent=TRUE)
 	if (inherits(tf, "try-error")) {
@@ -156,6 +164,8 @@ Deriv <- function(f, x=if (is.function(f)) names(formals(f)) else all.vars(if (i
 	}
 	# clean dsym env
 	rm(list=ls(dsym), envir=dsym)
+	if (is.null(env))
+		env <- .GlobalEnv
 	if (is.null(x)) {
 		# primitive function
 		fch <- deparse(substitute(f))
@@ -164,8 +174,6 @@ Deriv <- function(f, x=if (is.function(f)) names(formals(f)) else all.vars(if (i
 		if ("..." %in% x) {
 			stop(sprintf("Undefined list of arguments for %s()", fch))
 		}
-		if (is.null(env))
-			env <- .GlobalEnv
 		rule <- drule[[fch]]
 		if (is.null(rule) && !fch %in% dlin) {
 			stop(sprintf("Undefined rule for '%s()' differentiation", fch))
@@ -187,15 +195,17 @@ Deriv <- function(f, x=if (is.function(f)) names(formals(f)) else all.vars(if (i
 		format1(res)
 	} else if (is.function(f)) {
 		b <- body(f)
-		if (is.call(b) && b[[1]] == as.symbol(".Internal")) {
+		if ((is.call(b) && (b[[1]] == as.symbol(".Internal") || b[[1]] == as.symbol(".External"))) || (is.null(b) && is.primitive(f))) {
 			fch <- deparse(substitute(f))
 			if (fch %in% dlin || !is.null(drule[[fch]])) {
-				res <- Deriv_(as.call(c(substitute(f), lapply(x, as.symbol))), x, env, use.D)
+				arg <- lapply(names(formals(args(f))), as.symbol)
+				acall <- as.call(c(as.symbol(fch), arg))
+				res <- Deriv_(acall, x, env, use.D)
 				if (cache.exp)
 					res <- Cache(res)
-				as.function(c(formals(f), res), envir=env)
+				as.function(c(formals(args(f)), res), envir=env)
 			} else {
-				stop(sprintf("Internal function '%s()' is not in derivative table.", fch))
+				stop(sprintf("Internal or external function '%s()' is not in derivative table.", fch))
 			}
 		} else {
 			res <- Deriv_(b, x, env, use.D)
@@ -239,12 +249,12 @@ Deriv_ <- function(st, x, env, use.D) {
 	# Make x scalar and wrap results in a c() call if length(x) > 1
 	if (length(x) > 1) {
 		# many variables => recursive call on single name
-		res <- lapply(x, function(xi) Deriv_(st, xi, env, use.D))
+		res <- lapply(x, function(xi) {rm(list=ls(dsym), envir=dsym); Deriv_(st, xi, env, use.D)})
 		names(res) <- x;
 		return(as.call(c(as.symbol("c"), res)))
 	}
 	# differentiate R statement 'st' (a call, or a symbol or numeric) by a name in 'x'
-	if (is.unumeric(st)) {
+	if (is.conuloch(st)) {
 		return(0)
 	} else if (is.symbol(st)) {
 		stch <- as.character(st)
@@ -255,12 +265,6 @@ Deriv_ <- function(st, x, env, use.D) {
 		} else {
 			return(dsym[[stch]])
 		}
-	} else if ((is.uminus(st) || is.uplus(st)) && is.symbol(st[[2]])) {
-		if (as.character(st[[2]]) == x) {
-			return(if (is.uminus(st)) -1 else 1)
-		} else {
-			return(0)
-		}
 	} else if (is.call(st)) {
 #browser()
 		stch <- as.character(st[[1]])
@@ -268,10 +272,11 @@ Deriv_ <- function(st, x, env, use.D) {
 		if (stch %in% dlin) {
 			# linear case
 			# differentiate all arguments then pass them to the function
-			dargs <- lapply(as.list(st)[-1], function(a) Deriv_(a, x, env, use.D))
+			dargs <- lapply(args, Deriv_, x, env, use.D)
 			return(Simplify_(as.call(c(st[[1]], dargs))))
 		}
 		nb_args=length(st)-1
+		# special cases: out of rule table or args(stch) -> NULL
 		if (stch == "{") {
 #browser()
 			# AD differentiation
@@ -279,7 +284,7 @@ Deriv_ <- function(st, x, env, use.D) {
 			for (a in args) {
 				if (is.call(a) && (a[[1]] == as.symbol("<-") || a[[1]] == as.symbol("="))) {
 					if (!is.symbol(a[[2]]))
-						stop(sprintf("In AD mode, don't know to deal with a non symbol '%s' at lhs", format1(a[[2]])))
+						stop(sprintf("In AD mode, don't know how to deal with a non symbol '%s' at lhs", format1(a[[2]])))
 					res <- append(res, a)
 					de_a <- Deriv_(a[[3]], x, env, use.D)
 					ach <- as.character(a[[2]])
@@ -297,8 +302,18 @@ Deriv_ <- function(st, x, env, use.D) {
 				}
 			}
 			return(as.call(res))
+		} else if (is.uminus(st)) {
+			return(Simplify(call("-", Deriv_(st[[2]], x, env, use.D))))
+		} else if (stch == "(") {
+			return(Simplify(Deriv_(st[[2]], x, env, use.D)))
+		} else if (stch == "if") {
+			return(if (nb_args == 2)
+				Simplify(call("if", st[[2]], Deriv_(st[[3]], x, env, use.D))) else
+				Simplify(call("if", st[[2]], Deriv_(st[[3]], x, env, use.D),
+					Deriv_(st[[4]], x, env, use.D))))
 		}
-		if (is.null(drule[[stch]])) {
+		rule <- drule[[stch]]
+		if (is.null(rule)) {
 			# no derivative rule for this function
 			# try to get the body and differentiate it
 			ff <- get(stch, mode="function", envir=env)
@@ -309,33 +324,48 @@ Deriv_ <- function(st, x, env, use.D) {
 			mc <- match.call(ff, st)
 			st <- Simplify_(do.call("substitute", list(bf, as.list(mc)[-1])))
 			return(Deriv_(st, x, env, use.D))
-		} else if (is.null(drule[[stch]][[nb_args]]) && !use.D) {
-			stop(sprintf("Don't know how to differentiate function or operator '%s' when it is called with %s arguments", stch, nb_args))
 		}
 		# there is a rule!
 		if (use.D) {
 			return(Simplify(D(st, x)))
 		}
-		# prepare replacement list ._1 -> first argument, ._d1 -> derivative of the first argument and so on
-		args <- as.list(st)[-1]
-		names(args) <- paste("._", seq_len(nb_args), sep="")
+		# prepare replacement list
+		da <- args(stch)
+		mc <- as.list(match.call(definition=da, call=st))[-1]
+		da <- as.list(da)
+		da <- da[-length(da)] # all declared arguments with default values
+		aa <- modifyList(da, mc) # all arguments with actual values
+		# actualize the rule with actual arguments
+		rule <- lapply(rule, function(r) do.call("substitute", list(r, aa)))
+#browser()		
 		# which arguments have to be differentiated?
-		dstr <- format1(drule[[stch]][[nb_args]])
-		ma <- gregexpr("\\._d[1-9][0-9]*", dstr)
-		dgrep <- substring(dstr, ma[[1]], ma[[1]]+attr(ma[[1]], "match.length")-1)
-		if (any(nchar(dgrep) != 0)) {
-			dnum <- as.integer(sub("._d", "", unique(dgrep)))
-			dargs <- lapply(args[dnum], Deriv_, x, env, use.D)
-#cat("st=", format1(st), "\n", sep="")
-#cat("stch=", stch, "\n", sep="")
-#cat("dgrep=", dgrep, "\n", sep=", ")
-#cat("dstr=", dstr, "\n", sep="")
-#cat("dnum=", dnum, "\n", sep=", ")
-			names(dargs) <- paste("._d", dnum, sep="")
-		} else {
-			dargs <- NULL
+		iad <- which(!sapply(rule, is.null))
+		rule <- rule[iad]
+		lsy <- ls(dsym, all.names=TRUE)
+		if (!any(names(which(sapply(mc, function(it) {av <- all.vars(it); any(x == av) || any(av %in% lsy)}))) == names(rule))) {
+			#warning(sprintf("A call %s cannot be differentiated by the argument '%s'", format1(st), x))
+			return(0)
 		}
-		return(Simplify_(do.call("substitute", list(drule[[stch]][[nb_args]], c(args, dargs)))))
+		# rules and dargs are ordered by mc names
+		rule <- rule[names(mc)]
+		dargs <- lapply(mc, Deriv_, x, env, use.D)
+		ize <- sapply(dargs, `==`, 0)
+		dargs <- dargs[!ize]
+		rule <- rule[!ize]
+		if (length(rule) == 0) {
+			return(0)
+		}
+		
+		# apply chain rule where needed
+		ione <- sapply(dargs, `==`, 1)
+		imone <- sapply(dargs, `==`, -1)
+		for (i in seq_along(rule)[!(ione|imone)]) {
+			rule[[i]] <- call("*", dargs[[i]], rule[[i]])
+		}
+		for (i in seq_along(rule)[imone]) {
+			rule[[i]] <- call("-", rule[[i]])
+		}
+		return(Simplify(li2sum(rule)))
 	} else if (is.function(st)) {
 		# differentiate its body if can get it
 		args <- as.list(st)[-1]
@@ -346,82 +376,70 @@ Deriv_ <- function(st, x, env, use.D) {
 		st <- do.call("substitute", list(body(ff), args))
 		Deriv_(st, x, env, use.D)
 	} else {
-		stop("Invalid type of 'st' argument. It must be numeric, symbol or a call.")
+		stop("Invalid type of 'st' argument. It must be constant, symbol or a call.")
 	}
 }
 
+#' @rdname Deriv
+qlist <- function(...) {
+	# substitute arguments and return the list
+	mc <- match.call()
+	as.list(mc)[-1]
+}
 drule <- new.env()
 dsym <- new.env()
 
 # linear functions, i.e. d(f(x))/dx == f(d(arg)/dx)
 dlin=c("+", "-", "c", "t", "sum", "cbind", "rbind")
 
-# first item in the list correspond to a call with one argument
-# second (if any) for two, third for three. NULL means that with
-# this number of argument this function can not be called
-drule[["("]] <- list(quote(._d1)) # (._1) => omit paranthesis
-# linear arithmetics are already handled by dlin
-# exception is maid for unitary plus (it is just omitted)
-#drule[["+"]] <- list(quote(._d1), quote(._d1+._d2)) # +._1, ._1+._2
-#drule[["-"]] <- list(quote(-._d1), quote(._d1-._d2)) # -._1, ._1-._2
-# arithmetic non linear rules
-drule[["*"]] <- list(NULL, quote(._d1*._2+._1*._d2)) # ._1*._2
-drule[["/"]] <- list(NULL, quote(._d1/._2-._1*._d2/._2^2)) # ._1*._2
-# power functions
-drule[["^"]] <- list(NULL, quote(._d1*._2*._1^(._2-1)+._d2*log(._1)*._1^._2)) # ._1^._2
-# example of recursive call
-#drule[["sqrt"]] <- list(Deriv(call("^", as.symbol("._1"), 0.5), "._1", NULL, use.D)) # sqrt(._1)
-# but we prefer a sqrt() formula
-drule[["sqrt"]] <- list(quote(0.5*._d1/sqrt(._1)))
-drule[["log"]] <- list(quote(._d1/._1), quote(._d1/(._1*log(._2))- ._d2*log(._2, ._1)/(._2*log(._2)))) # log(._1), log(._1, b)
+
+drule[["*"]] <- qlist(e1=e2, e2=e1)
+drule[["^"]] <- qlist(e1=e2*e1^(e2-1), e2=e1^e2*log(e1))
+drule[["/"]] <- qlist(e1=1/e2, e2=-e1/e2^2)
+drule[["sqrt"]] <- qlist(x=0.5/sqrt(x))
+drule[["log"]] <- qlist(x=1/(x*log(base)), base=-log(base, x)/(base*log(base)))
 drule[["logb"]] <- drule[["log"]]
-drule[["log2"]] <- list(quote(._d1/(._1*log(2))))
-drule[["log10"]] <- list(quote(._d1/(._1*log(10))))
-drule[["log1p"]] <- list(quote(._d1/(._1+1)))
-drule[["exp"]] <- list(quote(._d1*exp(._1)))
-drule[["expm1"]] <- list(quote(._d1*exp(._1)))
+drule[["log2"]] <- qlist(x=1/(x*log(2)))
+drule[["log10"]] <- qlist(x=1/(x*log(10)))
+drule[["log1p"]] <- qlist(x=1/(x+1))
+drule[["exp"]] <- qlist(x=exp(x))
+drule[["expm1"]] <- qlist(x=exp(x))
 # trigonometric
-drule[["sin"]] <- list(quote(._d1*cos(._1)))
-drule[["cos"]] <- list(quote(-._d1*sin(._1)))
-drule[["tan"]] <- list(quote(._d1/cos(._1)^2))
-drule[["asin"]] <- list(quote(._d1/sqrt(1-._1^2)))
-drule[["acos"]] <- list(quote(-._d1/sqrt(1-._1^2)))
-drule[["atan"]] <- list(quote(._d1/(1+._1^2)))
-drule[["atan2"]] <- list(NULL, quote((._d1*._2-._d2*._1)/(._1^2+._2^2)))
-drule[["sinpi"]] <- list(quote(pi*._d1*cospi(._1)))
-drule[["cospi"]] <- list(quote(-pi*._d1*sinpi(._1)))
-drule[["tanpi"]] <- list(quote(pi*._d1/cospi(._1)^2))
+drule[["sin"]] <- qlist(x=cos(x))
+drule[["cos"]] <- qlist(x=-sin(x))
+drule[["tan"]] <- qlist(x=1/cos(x)^2)
+drule[["asin"]] <- qlist(x=1/sqrt(1-x^2))
+drule[["acos"]] <- qlist(x=-1/sqrt(1-x^2))
+drule[["atan"]] <- qlist(x=1/(1+x^2))
+drule[["atan2"]] <- qlist(y=x/(x^2+y^2), x=-y/(x^2+y^2))
+drule[["sinpi"]] <- qlist(x=pi*cospi(x))
+drule[["cospi"]] <- qlist(x=-pi*sinpi(x))
+drule[["tanpi"]] <- qlist(x=pi/cospi(x)^2)
 # hyperbolic
-drule[["sinh"]] <- list(quote(._d1*cosh(._1)))
-drule[["cosh"]] <- list(quote(._d1*sinh(._1)))
-drule[["tanh"]] <- list(quote(._d1*(1-tanh(._1)^2)))
-drule[["asinh"]] <- list(quote(._d1/sqrt(._1^2+1)))
-drule[["acosh"]] <- list(quote(._d1/sqrt(._1^2-1)))
-drule[["atanh"]] <- list(quote(._d1/(1-._1^2)))
-# code control
-drule[["if"]] <- list(NULL, quote(if (._1) ._d2), quote(if (._1) ._d2 else ._d3))
+drule[["sinh"]] <- qlist(x=cosh(x))
+drule[["cosh"]] <- qlist(x=sinh(x))
+drule[["tanh"]] <- qlist(x=(1-tanh(x)^2))
+drule[["asinh"]] <- qlist(x=1/sqrt(x^2+1))
+drule[["acosh"]] <- qlist(x=1/sqrt(x^2-1))
+drule[["atanh"]] <- qlist(x=1/(1-x^2))
 # sign depending functions
-drule[["abs"]] <- list(quote(._d1*sign(._1)))
-drule[["sign"]] <- list(0)
+drule[["abs"]] <- qlist(x=sign(x))
+drule[["sign"]] <- qlist(x=0)
 # special functions
-drule[["besselI"]] <- list(NULL,
-	quote(if (._2 == 0) ._d1*besselI(._1, 1) else 0.5*._d1*(besselI(._1, ._2-1) + besselI(._1, ._2+1))),
-	quote((if (._2 == 0) ._d1*besselI(._1, 1, ._3) else 0.5*._d1*(besselI(._1, ._2-1, ._3) + besselI(._1, ._2+1, ._3)))-if (._3) besselI(._1, ._2, TRUE) else 0)
-)
-drule[["besselK"]] <- list(NULL,
-	quote(if (._2 == 0) -._d1*besselK(._1, 1) else -0.5*._d1*(besselK(._1, ._2-1) + besselK(._1, ._2+1))),
-	quote((if (._2 == 0) -._d1*besselK(._1, 1, ._3) else -0.5*._d1*(besselK(._1, ._2-1, ._3) + besselK(._1, ._2+1, ._3)))+if (._3) besselK(._1, ._2, TRUE) else 0)
-)
-drule[["besselJ"]] <- list(NULL,
-	quote(if (._2 == 0) -._d1*besselJ(._1, 1) else 0.5*._d1*(besselJ(._1, ._2-1) - besselJ(._1, ._2+1)))
-)
-drule[["besselY"]] <- list(NULL,
-	quote(if (._2 == 0) -._d1*besselY(._1, 1) else 0.5*._d1*(besselY(._1, ._2-1) - besselY(._1, ._2+1)))
-)
-drule[["gamma"]] <- list(quote(._d1*gamma(._1)*digamma(._1)))
-drule[["lgamma"]] <- list(quote(._d1*digamma(._1)))
-drule[["digamma"]] <- list(quote(._d1*trigamma(._1)))
-drule[["trigamma"]] <- list(quote(._d1*psigamma(._1, 2L)))
-drule[["psigamma"]] <- list(quote(._d1*psigamma(._1, 1L)), quote(._d1*psigamma(._1, ._2+1L)))
-drule[["beta"]] <- list(NULL, quote(beta(._1, ._2)*(._d1*digamma(._1)+._d2*digamma(._2)-digamma(._1+._2)*(._d1+._d2))))
-drule[["lbeta"]] <- list(NULL, quote(._d1*digamma(._1)+._d2*digamma(._2)-digamma(._1+._2)*(._d1+._d2)))
+drule[["besselI"]] <- qlist(x=(if (nu == 0) besselI(x, 1, expon.scaled) else 0.5*(besselI(x, nu-1, expon.scaled) + besselI(x, nu+1, expon.scaled)))-if (expon.scaled) besselI(x, nu, TRUE) else 0, nu=NULL, expon.scaled=NULL)
+drule[["besselK"]] <- qlist(x=(if (nu == 0) -besselK(x, 1, expon.scaled) else -0.5*(besselK(x, nu-1, expon.scaled) + besselK(x, nu+1, expon.scaled)))+if (expon.scaled) besselK(x, nu, TRUE) else 0)
+drule[["besselJ"]] <- qlist(x=if (nu == 0) -besselJ(x, 1) else 0.5*(besselJ(x, nu-1) - besselJ(x, nu+1)), nu=NULL)
+drule[["besselY"]] <- qlist(x=if (nu == 0) -besselY(x, 1) else 0.5*(besselY(x, nu-1) - besselY(x, nu+1)), nu=NULL)
+drule[["gamma"]] <- qlist(x=gamma(x)*digamma(x))
+drule[["lgamma"]] <- qlist(x=digamma(x))
+drule[["digamma"]] <- qlist(x=trigamma(x))
+drule[["trigamma"]] <- qlist(x=psigamma(x, 2L))
+drule[["psigamma"]] <- qlist(x=psigamma(x, deriv+1L), deriv=NULL)
+drule[["beta"]] <- qlist(a=beta(a, b)*(digamma(a)-digamma(a+b)), b=beta(a, b)*(digamma(b)-digamma(a+b)))
+drule[["lbeta"]] <- qlist(a=digamma(a)-digamma(a+b), b=digamma(b)-digamma(a+b))
+# probability densities
+drule[["dbinom"]] <- qlist(x=NULL, size=NULL, log=NULL, prob=if (size == 0) -x*(1-prob)^(x-1) else if (x == size) size*prob^(size-1) else (size-x*prob)*(x-size+1)*dbinom(x, size-1, prob)/(1-prob)^2/(if (log) dbinom(x, size, prob) else 1))
+drule[["dnorm"]] <- qlist(x=-(x-mean)/sd^2*if (log) 1 else dnorm(x, mean, sd),
+	mean=(x-mean)/sd^2*if (log) 1 else dnorm(x, mean, sd),
+	sd=(((x - mean)/sd)^2 - 1)/sd * if (log) 1 else dnorm(x, mean, sd),
+	log=NULL)
