@@ -48,6 +48,7 @@ test_that("elementary functions", {
    expect_equal(Deriv(quote((x)), "x"), 1)
    expect_equal_deriv(x**2, 2*x)
    expect_equal_deriv(x**n, n*x^(n-1))
+   expect_equal_deriv(2**x, 0.693147180559945 * 2^x)
    expect_equal_deriv(sin(x), cos(x))
    expect_equal_deriv(cos(x), -sin(x))
    expect_equal_deriv(tan(x), 1/cos(x)^2)
@@ -55,6 +56,7 @@ test_that("elementary functions", {
    expect_equal_deriv(acos(x), -(1/sqrt(1 - x^2)))
    expect_equal_deriv(atan(x), 1/(1+x^2))
    expect_equal_deriv(atan2(x, y), y/(x^2+y^2))
+   expect_equal_deriv(atan2(0.5, x), -(0.5/(0.25 + x^2)))
    expect_equal_deriv(exp(x), exp(x))
    expect_equal_deriv(expm1(x), exp(x))
    expect_equal_deriv(log(x), 1/x)
@@ -72,22 +74,22 @@ test_that("special functions", {
    expect_equal_deriv(beta(x, y), beta(x, y) * (digamma(x) - digamma(x + y)))
    expect_equal_deriv(beta(x, y), beta(x, y) * (digamma(y) - digamma(x + y)), "y")
    expect_equal_deriv(besselI(x, 0), besselI(x, 1))
-   expect_equal_deriv(besselI(x, 0, FALSE), besselI(x, 1, FALSE))
+   expect_equal_deriv(besselI(x, 0, FALSE), besselI(x, 1))
    expect_equal_deriv(besselI(x, 0, TRUE), besselI(x, 1, TRUE)-besselI(x, 0, TRUE))
    expect_equal_deriv(besselI(x, 1), 0.5 * (besselI(x, 0) + besselI(x, 2)))
-   expect_equal_deriv(besselI(x, 1, FALSE), 0.5 * (besselI(x, 0, FALSE) + besselI(x, 2, FALSE)))
+   expect_equal_deriv(besselI(x, 1, FALSE), 0.5 * (besselI(x, 0) + besselI(x, 2)))
    expect_equal_deriv(besselI(x, 1, TRUE), 0.5 * (besselI(x, 0, TRUE) + besselI(x, 2, TRUE))-besselI(x, 1, TRUE))
-   expect_equal_deriv(besselI(x, n), if (n == 0) besselI(x, 1) else 0.5 * (besselI(x, n - 1) + besselI(x, n + 1)))
-   expect_equal_deriv(besselI(x, n, TRUE), (if (n == 0) besselI(x, 1, TRUE) else 0.5 * (besselI(x, n - 1, TRUE) + besselI(x, n + 1, TRUE)))-besselI(x, n, TRUE))
+   expect_equal_deriv(besselI(x, n), if (n == 0) besselI(x, 1) else 0.5 * (besselI(x, n + 1) + besselI(x, n - 1)))
+   expect_equal_deriv(besselI(x, n, TRUE), (if (n == 0) besselI(x, 1, TRUE) else 0.5 * (besselI(x, n + 1, TRUE) + besselI(x, n - 1, TRUE)))-besselI(x, n, TRUE))
    expect_equal_deriv(besselK(x, 0), -besselK(x, 1))
-   expect_equal_deriv(besselK(x, 0, FALSE), -besselK(x, 1, FALSE))
+   expect_equal_deriv(besselK(x, 0, FALSE), -besselK(x, 1))
    expect_equal_deriv(besselK(x, 0, TRUE), besselK(x, 0, TRUE)-besselK(x, 1, TRUE))
    expect_equal_deriv(besselK(x, 1), -(0.5 * (besselK(x, 0) + besselK(x, 2))))
-   expect_equal_deriv(besselK(x, 1, FALSE), -(0.5 * (besselK(x, 0, FALSE) + besselK(x, 2, FALSE))))
+   expect_equal_deriv(besselK(x, 1, FALSE), -(0.5 * (besselK(x, 0) + besselK(x, 2))))
    expect_equal_deriv(besselK(x, 1, TRUE), besselK(x, 1, TRUE)-0.5 * (besselK(x, 0, TRUE) + besselK(x, 2, TRUE)))
-   expect_equal_deriv(besselK(x, n), if (n == 0) -besselK(x, 1) else -(0.5 * (besselK(x, n - 1) + besselK(x, n + 1))))
-   expect_equal_deriv(besselK(x, n, FALSE), if (n == 0) -besselK(x, 1, FALSE) else -(0.5 * (besselK(x, n - 1, FALSE) + besselK(x, n + 1, FALSE))))
-   expect_equal_deriv(besselK(x, n, TRUE), (if (n == 0) -besselK(x, 1, TRUE) else -(0.5 * (besselK(x, n - 1, TRUE) + besselK(x, n + 1, TRUE))))+besselK(x, n, TRUE))
+   expect_equal_deriv(besselK(x, n), if (n == 0) -besselK(x, 1) else -(0.5 * (besselK(x, n + 1) + besselK(x, n - 1))))
+   expect_equal_deriv(besselK(x, n, FALSE), if (n == 0) -besselK(x, 1) else -(0.5 * (besselK(x, n + 1) + besselK(x, n - 1))))
+   expect_equal_deriv(besselK(x, n, TRUE), besselK(x, n, TRUE)+if (n == 0) -besselK(x, 1, TRUE) else -(0.5 * (besselK(x, n + 1, TRUE) + besselK(x, n - 1, TRUE))))
    expect_equal_deriv(besselJ(x, 0), -besselJ(x, 1))
    expect_equal_deriv(besselJ(x, 1), 0.5 * (besselJ(x, 0) - besselJ(x, 2)))
    expect_equal_deriv(besselJ(x, n), if (n == 0) -besselJ(x, 1) else 0.5 * (besselJ(x, n - 1) - besselJ(x, n + 1)))
@@ -105,6 +107,10 @@ test_that("special functions", {
    expect_equal_deriv(lbeta(x, y), digamma(x) - digamma(x + y))
    expect_equal_deriv(lbeta(x, y), digamma(y) - digamma(x + y), "y")
 })
+test_that("probability densities", {
+   expect_equal_deriv(dbinom(5,3,x), 3 * ((3 - 5 * x) * dbinom(5, 2, x)/(1 - x)^2))
+   expect_equal_deriv(dnorm(x, m=0.5), -(dnorm(x, 0.5, 1) * (x - 0.5)))
+})
 test_that("chain rule: multiply by a const", {
    expect_equal_deriv(a*x, a)
    expect_equal_deriv((a*x)**2, 2*(a^2*x))
@@ -117,7 +123,9 @@ test_that("chain rule: multiply by a const", {
 })
 test_that("particular cases", {
    expect_equal_deriv(log(x, x), 0)
-   expect_equal_deriv(x^n+sin(n*x), n * (x^(n - 1) + cos(n * x)))
+   expect_equal_deriv(x^n+sin(n*x), n * (cos(n * x) + x^(n - 1)))
+   expect_equal_deriv(x*(1-x), 1-2*x)
+   expect_equal_deriv(x^x, x^x+x^x*log(x))
 })
 
 # test AD and caching
