@@ -48,7 +48,10 @@ test_that("abs simplifications", {
 })
 test_that("factorizations", {
    expect_equal_lang(a+b, a+b) # no change must occur
+   expect_equal_lang(3-5*x, 3-5*x) # no change must occur
    expect_equal_lang(a*a+b*a, a*(a+b))
    expect_equal_lang(a^2+b*a^3, a^2*(1+a*b))
    expect_equal_lang(a^2/c**5+b*a^3/d/c**3, a^2*(1/c^2+a*b/d)/c^3)
+   expect_equal_lang(a+1-a, 1)
+   expect_equal_lang(1+x-1, x)
 })
